#![no_std]
#![no_main]

use atsame54_xpro as bsp;
use bsp::hal;

#[cfg(not(feature = "use_semihosting"))]
use panic_halt as _;
#[cfg(feature = "use_semihosting")]
use panic_semihosting as _;

use bsp::{entry, gpio::v2::Pins, gpio::v2::ReadableOutput};
use hal::clock::GenericClockController;
use hal::delay::Delay;
use hal::pac::{CorePeripherals, Peripherals};
use hal::prelude::*;

use hal::watchdog::{ Watchdog, WatchdogTimeout };

use bsp::hal::sercom::{
    v2::{
        uart, uart::BaudMode, uart::Oversampling
        }
    };
use embedded_hal::serial::Write;


#[entry]
fn main() -> ! {
    let mut peripherals = Peripherals::take().unwrap();
    let core = CorePeripherals::take().unwrap();
    let mut clocks = GenericClockController::with_internal_32kosc(
        peripherals.GCLK,
        &mut peripherals.MCLK,
        &mut peripherals.OSC32KCTRL,
        &mut peripherals.OSCCTRL,
        &mut peripherals.NVMCTRL,
    );
    let mut delay = Delay::new(core.SYST, &mut clocks);
    delay.delay_ms(400u16);

    let pins = Pins::new(peripherals.PORT);
    let mut led = pins.pc18.into_mode::<ReadableOutput>();

    let gclk0 = clocks.gclk0();
    let clock = &clocks.sercom2_core(&gclk0).unwrap();
    let pads = uart::Pads::default().rx(pins.pb24).tx(pins.pb25);
    let mut uart_cfg = uart::Config::new(&peripherals.MCLK, peripherals.SERCOM2, pads, clock.freq());
    uart_cfg.set_baud(115200.hz(), BaudMode::Arithmetic(Oversampling::Bits8));
    let uart = uart_cfg.enable();

    let (_, mut tx) = uart.split();

    let s_off: &str = "off\r\n";
    let s_on: &str = "on\r\n";

    let mut wdt = Watchdog::new(peripherals.WDT);
    wdt.start(WatchdogTimeout::Cycles256 as u8);

    loop {
        delay.delay_ms(200u8);
        wdt.feed();
        led.set_high().unwrap();
        for b in s_off.bytes() {
            nb::block!(tx.write(b)).unwrap();
        }
            delay.delay_ms(200u8);
        wdt.feed();
        led.set_low().unwrap();
        for b in s_on.bytes() {
            nb::block!(tx.write(b)).unwrap();
        }
    }
}
